@@ -147,10 +147,9 @@ def main():
       env[var_name] = os.environ.get(var_name, "") + " external_symbolizer_path=" + symbolizer_path
 
   # Add environment variables for Java dependencies. These environment variables
-  # are used in mini_hms.cc and mini_sentry.cc.
+  # are used in mini_hms.cc and mini_ranger.cc.
   env['HIVE_HOME'] = glob.glob(os.path.join(ROOT, "thirdparty/src/hive-*"))[0]
   env['HADOOP_HOME'] = glob.glob(os.path.join(ROOT, "thirdparty/src/hadoop-*"))[0]
-  env['SENTRY_HOME'] = glob.glob(os.path.join(ROOT, "thirdparty/src/sentry-*"))[0]
   env['RANGER_HOME'] = glob.glob(os.path.join(ROOT, "thirdparty/src/ranger-*"))[0]
   env['JAVA_HOME'] = glob.glob("/usr/lib/jvm/java-1.8.0-*")[0]
 
