@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 package org.apache.kudu.backup
 
 import scala.concurrent.forkjoin.ForkJoinPool
@@ -96,7 +97,7 @@ object KuduBackup {
     io.writeTableMetadata(tableMetadata, metadataPath)
   }
 
-  def run(options: BackupOptions, session: SparkSession): Int = {
+  def run(options: BackupOptions, session: SparkSession): Boolean = {
     // Set the job group for all the spark backup jobs.
     // Note: The job description will be overridden by each Kudu table job.
     session.sparkContext.setJobGroup(s"Kudu Backup @ ${options.toMs}", "Kudu Backup")
@@ -139,7 +140,7 @@ object KuduBackup {
       val backupResult = Try(doBackup(tableName, context, session, io, options, backupMap))
       backupResult match {
         case Success(()) =>
-          log.info(s"Successfully backed up up table $tableName")
+          log.info(s"Successfully backed up table $tableName")
         case Failure(ex) =>
           if (options.numParallelBackups == 1 && options.failOnFirstError)
             throw ex
@@ -155,10 +156,7 @@ object KuduBackup {
         log.error(
           s"Failed to back up table $tableName: Look back in the logs for the full exception. Error: ${ex.toString}")
     }
-    if (backupResults.exists(_._2.isFailure))
-      1
-    else
-      0
+    !backupResults.exists(_._2.isFailure)
   }
 
   def main(args: Array[String]): Unit = {
@@ -171,6 +169,11 @@ object KuduBackup {
       .appName("Kudu Table Backup")
       .getOrCreate()
 
-    System.exit(run(options, session))
+    val isRunSuccessful: Boolean = run(options, session)
+    if (!isRunSuccessful) {
+      throw new RuntimeException("Kudu Table Backup application failed")
+    }
+
+    session.stop()
   }
 }
