@@ -29,9 +29,7 @@ package org.apache.kudu.client;
 import java.io.DataInput;
 import java.io.DataOutput;
 import java.io.IOException;
-import java.lang.reflect.Field;
-import java.lang.reflect.InvocationTargetException;
-import java.lang.reflect.Method;
+import java.io.Serializable;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.nio.ByteBuffer;
@@ -40,9 +38,9 @@ import java.util.BitSet;
 import java.util.Comparator;
 
 import com.google.common.io.BaseEncoding;
+import io.netty.buffer.ByteBuf;
+import io.netty.util.CharsetUtil;
 import org.apache.yetus.audience.InterfaceAudience;
-import org.jboss.netty.buffer.ChannelBuffer;
-import org.jboss.netty.util.CharsetUtil;
 
 import org.apache.kudu.util.DecimalUtil;
 import org.apache.kudu.util.Slice;
@@ -401,7 +399,7 @@ public final class Bytes {
    * @param buf The buffer to read from.
    * @return The integer read.
    */
-  static int readVarInt32(final ChannelBuffer buf) {
+  static int readVarInt32(final ByteBuf buf) {
     int result = buf.readByte();
     if (result >= 0) {
       return result;
@@ -660,7 +658,7 @@ public final class Bytes {
    */
   private static void reverseBytes(final byte[] b) {
     // Swaps the items until the mid-point is reached.
-    for(int i = 0; i < b.length / 2; i++) {
+    for (int i = 0; i < b.length / 2; i++) {
       byte temp = b[i];
       b[i] = b[b.length - i - 1];
       b[b.length - i - 1] = temp;
@@ -802,7 +800,7 @@ public final class Bytes {
         long longVal = getLong(b, offset);
         return BigDecimal.valueOf(longVal, scale);
       case DecimalUtil.DECIMAL128_SIZE:
-         BigInteger int128Val = getBigInteger(b, offset);
+        BigInteger int128Val = getBigInteger(b, offset);
         return new BigDecimal(int128Val, scale);
       default:
         throw new IllegalArgumentException("Unsupported decimal type size: " + size);
@@ -829,7 +827,8 @@ public final class Bytes {
    * @param offset The offset in the array to start writing at.
    * @throws IndexOutOfBoundsException if the byte array is too small.
    */
-  public static void setBigDecimal(final byte[] b, final BigDecimal n, int precision, final int offset) {
+  public static void setBigDecimal(final byte[] b, final BigDecimal n, int precision,
+                                   final int offset) {
     int size = DecimalUtil.precisionToSize(precision);
     BigInteger bigInt = n.unscaledValue();
     switch (size) {
@@ -980,28 +979,12 @@ public final class Bytes {
    * @param buf The (possibly {@code null}) buffer to pretty-print.
    * @return The buffer in a pretty-printed string.
    */
-  public static String pretty(final ChannelBuffer buf) {
+  public static String pretty(final ByteBuf buf) {
     if (buf == null) {
       return "null";
     }
-    byte[] array;
-    try {
-      if (buf.getClass() != ReplayingDecoderBuffer) {
-        array = buf.array();
-      } else if (RDB_buf != null) {  // Netty 3.5.1 and above.
-        array = ((ChannelBuffer) RDB_buf.invoke(buf)).array();
-      } else {  // Netty 3.5.0 and before.
-        final ChannelBuffer wrapped_buf = (ChannelBuffer) RDB_buffer.get(buf);
-        array = wrapped_buf.array();
-      }
-    } catch (UnsupportedOperationException e) {
-      return "(failed to extract content of buffer of type " +
-          buf.getClass().getName() + ')';
-    } catch (IllegalAccessException e) {
-      throw new AssertionError("Should not happen: " + e);
-    } catch (InvocationTargetException e) {
-      throw new AssertionError("Should not happen: " + e);
-    }
+    byte[] array = new byte[buf.readableBytes()];
+    buf.getBytes(buf.readerIndex(), array);
     return pretty(array);
   }
 
@@ -1018,52 +1001,6 @@ public final class Bytes {
     return sb.toString();
   }
 
-  // Ugly stuff
-  // ----------
-  // Background: when using ReplayingDecoder (which makes it easy to deal with
-  // unframed RPC responses), the ChannelBuffer we manipulate is in fact a
-  // ReplayingDecoderBuffer, a package-private class that Netty uses.  This
-  // class, for some reason, throws UnsupportedOperationException on its
-  // array() method.  This method is unfortunately the only way to easily dump
-  // the contents of a ChannelBuffer, which is useful for debugging or logging
-  // unexpected buffers.  An issue (NETTY-346) has been filed to get access to
-  // the buffer, but the resolution was useless: instead of making the array()
-  // method work, a new internalBuffer() method was added on ReplayingDecoder,
-  // which would require that we keep a reference on the ReplayingDecoder all
-  // along in order to properly convert the buffer to a string.
-  // So we instead use ugly reflection to gain access to the underlying buffer
-  // while taking into account that the implementation of Netty has changed
-  // over time, so depending which version of Netty we're working with, we do
-  // a different hack.  Yes this is horrible, but it's for the greater good as
-  // this is what allows us to debug unexpected buffers when deserializing RPCs
-  // and what's more important than being able to debug unexpected stuff?
-  private static final Class<?> ReplayingDecoderBuffer;
-  private static final Field RDB_buffer;  // For Netty 3.5.0 and before.
-  private static final Method RDB_buf;    // For Netty 3.5.1 and above.
-
-  static {
-    try {
-      ReplayingDecoderBuffer = Class.forName("org.jboss.netty.handler.codec." +
-          "replay.ReplayingDecoderBuffer");
-      Field field = null;
-      try {
-        field = ReplayingDecoderBuffer.getDeclaredField("buffer");
-        field.setAccessible(true);
-      } catch (NoSuchFieldException e) {
-        // Ignore.  Field has been removed in Netty 3.5.1.
-      }
-      RDB_buffer = field;
-      if (field != null) {  // Netty 3.5.0 or before.
-        RDB_buf = null;
-      } else {
-        RDB_buf = ReplayingDecoderBuffer.getDeclaredMethod("buf");
-        RDB_buf.setAccessible(true);
-      }
-    } catch (Exception e) {
-      throw new RuntimeException("static initializer failed", e);
-    }
-  }
-
   // ---------------------- //
   // Comparing byte arrays. //
   // ---------------------- //
@@ -1076,7 +1013,9 @@ public final class Bytes {
   public static final MemCmp MEMCMP = new MemCmp();
 
   /** {@link Comparator} for non-{@code null} byte arrays.  */
-  private static final class MemCmp implements Comparator<byte[]> {
+  private static final class MemCmp implements Comparator<byte[]>, Serializable {
+
+    private static final long serialVersionUID = 914981342853419168L;
 
     private MemCmp() {  // Can't instantiate outside of this class.
     }
