@@ -68,6 +68,16 @@ public class TestAlterTable {
    * with the provided bounds.
    */
   private KuduTable createTable(List<Pair<Integer, Integer>> bounds) throws KuduException {
+    return createTable(bounds, null);
+  }
+
+  /**
+   * Creates a new table with two int columns, c0 and c1. c0 is the primary key.
+   * The table is hash partitioned on c0 into two buckets, and range partitioned
+   * with the provided bounds and the specified owner.
+   */
+  private KuduTable createTable(List<Pair<Integer, Integer>> bounds, String owner)
+      throws KuduException {
     // Create initial table with single range partition covering the entire key
     // space, and two hash buckets.
     ArrayList<ColumnSchema> columns = new ArrayList<>(1);
@@ -93,6 +103,10 @@ public class TestAlterTable {
       createOptions.addRangePartition(lower, upper);
     }
 
+    if (owner != null) {
+      createOptions.setOwner(owner);
+    }
+
     return client.createTable(tableName, schema, createOptions);
   }
 
@@ -564,4 +578,16 @@ public class TestAlterTable {
     Assert.assertTrue(thrown.getMessage()
             .contains("number of columns 11 is greater than the permitted maximum 10"));
   }
+
+  @Test
+  public void testAlterChangeOwner() throws Exception {
+    String originalOwner = "alice";
+    KuduTable table = createTable(ImmutableList.of(), originalOwner);
+    assertEquals(originalOwner, table.getOwner());
+
+    String newOwner = "bob";
+    client.alterTable(table.getName(), new AlterTableOptions().setOwner(newOwner));
+    table = client.openTable(table.getName());
+    assertEquals(newOwner, table.getOwner());
+  }
 }
