@@ -14,6 +14,7 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
+
 package org.apache.kudu.client;
 
 import static org.apache.kudu.Type.STRING;
@@ -42,14 +43,14 @@ import org.apache.kudu.test.KuduTestHarness;
 public class TestScannerMultiTablet {
   // Generate a unique table name
   private static final String TABLE_NAME =
-      TestScannerMultiTablet.class.getName()+"-"+System.currentTimeMillis();
+      TestScannerMultiTablet.class.getName() + "-" + System.currentTimeMillis();
 
   private static Schema schema = getSchema();
 
   /**
    * The timestamp after inserting the rows into the test table during setUp().
    */
-  private static long beforeWriteTimestamp;
+  private long beforeWriteTimestamp;
   private KuduTable table;
   private KuduClient client;
   private AsyncKuduClient asyncClient;
@@ -106,6 +107,27 @@ public class TestScannerMultiTablet {
     asyncClient = harness.getAsyncClient();
   }
 
+  private void validateResourceMetrics(ResourceMetrics resourceMetrics) {
+    assertTrue("queue_duration_nanos > 0",
+        resourceMetrics.getMetric("queue_duration_nanos") > 0L);
+    assertTrue("total_duration_nanos > 0",
+        resourceMetrics.getMetric("total_duration_nanos") > 0L);
+  }
+
+  // Test scanner resource metrics.
+  @Test(timeout = 100000)
+  public void testResourceMetrics() throws Exception {
+    // Scan one tablet and the whole table.
+    AsyncKuduScanner oneTabletScanner = getScanner("1", "1", "1", "4"); // Whole second tablet.
+    assertEquals(3, countRowsInScan(oneTabletScanner));
+    AsyncKuduScanner fullTableScanner = getScanner(null, null, null, null);
+    assertEquals(9, countRowsInScan(fullTableScanner));
+    // Both scans should take a positive amount of wait duration, total duration, cpu user and cpu
+    // system time
+    validateResourceMetrics(oneTabletScanner.getResourceMetrics());
+    validateResourceMetrics(fullTableScanner.getResourceMetrics());
+  }
+
   // Test various combinations of start/end row keys.
   @Test(timeout = 100000)
   public void testKeyStartEnd() throws Exception {
@@ -143,6 +165,7 @@ public class TestScannerMultiTablet {
 
   // Test mixing start/end row keys with predicates.
   @Test(timeout = 100000)
+  @SuppressWarnings("deprecation")
   public void testKeysAndPredicates() throws Exception {
     // Value that doesn't exist, predicates has primary column
     ColumnRangePredicate predicate = new ColumnRangePredicate(schema.getColumnByIndex(1));
@@ -439,6 +462,7 @@ public class TestScannerMultiTablet {
         exclusiveUpperBoundKeyOne, exclusiveUpperBoundKeyTwo, null);
   }
 
+  @SuppressWarnings("deprecation")
   private AsyncKuduScanner getScanner(String lowerBoundKeyOne,
                                       String lowerBoundKeyTwo,
                                       String exclusiveUpperBoundKeyOne,
