@@ -70,6 +70,8 @@ class DefaultSource
   val KEEP_ALIVE_PERIOD_MS = "kudu.keepAlivePeriodMs"
   val SPLIT_SIZE_BYTES = "kudu.splitSizeBytes"
   val HANDLE_SCHEMA_DRIFT = "kudu.handleSchemaDrift"
+  val USE_DRIVER_METADATA = "kudu.useDriverMetadata"
+  val SNAPSHOT_TIMESTAMP_MS = "kudu.snapshotTimestampMs"
 
   /**
    * A nice alias for the data source so that when specifying the format
@@ -183,7 +185,9 @@ class DefaultSource
     val keepAlivePeriodMs =
       parameters.get(KEEP_ALIVE_PERIOD_MS).map(_.toLong).getOrElse(defaultKeepAlivePeriodMs)
     val splitSizeBytes = parameters.get(SPLIT_SIZE_BYTES).map(_.toLong)
-
+    val useDriverMetadata =
+      parameters.get(USE_DRIVER_METADATA).map(_.toBoolean).getOrElse(defaultUseDriverMetadata)
+    val snapshotTimestampMs = parameters.get(SNAPSHOT_TIMESTAMP_MS).map(_.toLong)
     KuduReadOptions(
       batchSize,
       scanLocality,
@@ -191,7 +195,10 @@ class DefaultSource
       keepAlivePeriodMs,
       scanRequestTimeoutMs,
       /* socketReadTimeoutMs= */ None,
-      splitSizeBytes)
+      splitSizeBytes,
+      useDriverMetadata,
+      snapshotTimestampMs
+    )
   }
 
   private def getWriteOptions(parameters: Map[String, String]): KuduWriteOptions = {
