@@ -35,6 +35,7 @@ import org.apache.kudu.test.KuduTestHarness
 import org.apache.kudu.util.CharUtil
 import org.apache.kudu.util.DateUtil
 import org.apache.kudu.util.DecimalUtil
+import org.apache.kudu.util.HybridTimeUtil
 import org.apache.spark.sql.execution.datasources.LogicalRelation
 import org.apache.spark.sql.DataFrame
 import org.apache.spark.sql.SparkSession
@@ -52,6 +53,7 @@ trait KuduTestSuite extends JUnitSuite {
   var kuduContext: KuduContext = _
 
   val tableName: String = "test"
+  val owner: String = "testuser"
   val simpleTableName: String = "simple-test"
 
   lazy val schema: Schema = {
@@ -115,6 +117,7 @@ trait KuduTestSuite extends JUnitSuite {
       .setRangePartitionColumns(List("key").asJava)
       .addRangePartition(bottom, middle)
       .addRangePartition(middle, top)
+      .setOwner(owner)
       .setNumReplicas(1)
   }
 
@@ -205,6 +208,12 @@ trait KuduTestSuite extends JUnitSuite {
     rows
   }
 
+  def getLastPropagatedTimestampMs(): Long = {
+    HybridTimeUtil
+      .HTTimestampToPhysicalAndLogical(kuduClient.getLastPropagatedTimestamp)
+      .head / 1000
+  }
+
   def upsertRowsWithRowDataSize(
       targetTable: KuduTable,
       rowCount: Integer,
