@@ -144,6 +144,7 @@ class KuduClient::Data {
   //   'table_id'         The table unique id.
   //   'table_name'       The table unique name.
   //   'num_replicas'     The table replication factor.
+  //   'owner'            The owner of the table.
   //   'extra_configs'    The table's extra configuration properties.
   Status GetTableSchema(KuduClient* client,
                         const MonoTime& deadline,
@@ -153,6 +154,7 @@ class KuduClient::Data {
                         std::string* table_id,
                         std::string* table_name,
                         int* num_replicas,
+                        std::string* owner,
                         std::map<std::string, std::string>* extra_configs);
 
   Status InitLocalHostNames();
