@@ -17,7 +17,6 @@
 
 #include "kudu/client/schema.h"
 
-#include <cstdint>
 #include <memory>
 #include <ostream>
 #include <unordered_map>
@@ -35,13 +34,14 @@
 #include "kudu/common/schema.h"
 #include "kudu/common/types.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/util/char_util.h"
 #include "kudu/util/compression/compression.pb.h"
 #include "kudu/util/decimal_util.h"
 #include "kudu/util/slice.h"
+#include "kudu/util/string_case.h"
 
 DEFINE_bool(show_attributes, false,
             "Whether to show column attributes, including column encoding type, "
@@ -60,6 +60,7 @@ MAKE_ENUM_LIMITS(kudu::client::KuduColumnSchema::DataType,
                  kudu::client::KuduColumnSchema::BOOL);
 
 using std::string;
+using std::unique_ptr;
 using std::unordered_map;
 using std::vector;
 using strings::Substitute;
@@ -124,8 +125,10 @@ kudu::DataType ToInternalDataType(KuduColumnSchema::DataType type,
     case KuduColumnSchema::INT32: return kudu::INT32;
     case KuduColumnSchema::INT64: return kudu::INT64;
     case KuduColumnSchema::UNIXTIME_MICROS: return kudu::UNIXTIME_MICROS;
+    case KuduColumnSchema::DATE: return kudu::DATE;
     case KuduColumnSchema::FLOAT: return kudu::FLOAT;
     case KuduColumnSchema::DOUBLE: return kudu::DOUBLE;
+    case KuduColumnSchema::VARCHAR: return kudu::VARCHAR;
     case KuduColumnSchema::STRING: return kudu::STRING;
     case KuduColumnSchema::BINARY: return kudu::BINARY;
     case KuduColumnSchema::BOOL: return kudu::BOOL;
@@ -150,8 +153,10 @@ KuduColumnSchema::DataType FromInternalDataType(kudu::DataType type) {
     case kudu::INT32: return KuduColumnSchema::INT32;
     case kudu::INT64: return KuduColumnSchema::INT64;
     case kudu::UNIXTIME_MICROS: return KuduColumnSchema::UNIXTIME_MICROS;
+    case kudu::DATE: return KuduColumnSchema::DATE;
     case kudu::FLOAT: return KuduColumnSchema::FLOAT;
     case kudu::DOUBLE: return KuduColumnSchema::DOUBLE;
+    case kudu::VARCHAR: return KuduColumnSchema::VARCHAR;
     case kudu::STRING: return KuduColumnSchema::STRING;
     case kudu::BINARY: return KuduColumnSchema::BINARY;
     case kudu::BOOL: return KuduColumnSchema::BOOL;
@@ -167,7 +172,7 @@ KuduColumnSchema::DataType FromInternalDataType(kudu::DataType type) {
 ////////////////////////////////////////////////////////////
 
 KuduColumnTypeAttributes::KuduColumnTypeAttributes()
-    : data_(new Data(0, 0)) {
+    : data_(new Data(0, 0, 0)) {
 }
 
 KuduColumnTypeAttributes::KuduColumnTypeAttributes(const KuduColumnTypeAttributes& other)
@@ -176,7 +181,15 @@ KuduColumnTypeAttributes::KuduColumnTypeAttributes(const KuduColumnTypeAttribute
 }
 
 KuduColumnTypeAttributes::KuduColumnTypeAttributes(int8_t precision, int8_t scale)
-    : data_(new Data(precision, scale)) {
+    : data_(new Data(precision, scale, 0)) {
+}
+
+KuduColumnTypeAttributes::KuduColumnTypeAttributes(uint16_t length)
+    : data_(new Data(0, 0, length)) {
+}
+
+KuduColumnTypeAttributes::KuduColumnTypeAttributes(int8_t precision, int8_t scale, uint16_t length)
+    : data_(new Data(precision, scale, length)) {
 }
 
 KuduColumnTypeAttributes::~KuduColumnTypeAttributes() {
@@ -204,6 +217,60 @@ int8_t KuduColumnTypeAttributes::scale() const {
   return data_->scale;
 }
 
+uint16_t KuduColumnTypeAttributes::length() const {
+  return data_->length;
+}
+
+Status KuduColumnStorageAttributes::StringToEncodingType(
+    const string& encoding,
+    KuduColumnStorageAttributes::EncodingType* type) {
+  Status s;
+  string encoding_uc;
+  ToUpperCase(encoding, &encoding_uc);
+  if (encoding_uc == "AUTO_ENCODING") {
+    *type = KuduColumnStorageAttributes::AUTO_ENCODING;
+  } else if (encoding_uc == "PLAIN_ENCODING") {
+    *type = KuduColumnStorageAttributes::PLAIN_ENCODING;
+  } else if (encoding_uc == "PREFIX_ENCODING") {
+    *type = KuduColumnStorageAttributes::PREFIX_ENCODING;
+  } else if (encoding_uc == "RLE") {
+    *type = KuduColumnStorageAttributes::RLE;
+  } else if (encoding_uc == "DICT_ENCODING") {
+    *type = KuduColumnStorageAttributes::DICT_ENCODING;
+  } else if (encoding_uc == "BIT_SHUFFLE") {
+    *type = KuduColumnStorageAttributes::BIT_SHUFFLE;
+  } else if (encoding_uc == "GROUP_VARINT") {
+    *type = KuduColumnStorageAttributes::GROUP_VARINT;
+  } else {
+    s = Status::InvalidArgument(Substitute(
+        "encoding type $0 is not supported", encoding));
+  }
+  return s;
+}
+
+Status KuduColumnStorageAttributes::StringToCompressionType(
+    const string& compression,
+    KuduColumnStorageAttributes::CompressionType* type) {
+  Status s;
+  string compression_uc;
+  ToUpperCase(compression, &compression_uc);
+  if (compression_uc == "DEFAULT_COMPRESSION") {
+    *type = KuduColumnStorageAttributes::DEFAULT_COMPRESSION;
+  } else if (compression_uc == "NO_COMPRESSION") {
+    *type = KuduColumnStorageAttributes::NO_COMPRESSION;
+  } else if (compression_uc == "SNAPPY") {
+    *type = KuduColumnStorageAttributes::SNAPPY;
+  } else if (compression_uc == "LZ4") {
+    *type = KuduColumnStorageAttributes::LZ4;
+  } else if (compression_uc == "ZLIB") {
+    *type = KuduColumnStorageAttributes::ZLIB;
+  } else {
+    s = Status::InvalidArgument(Substitute(
+        "compression type $0 is not supported", compression));
+  }
+  return s;
+}
+
 ////////////////////////////////////////////////////////////
 // KuduColumnSpec
 ////////////////////////////////////////////////////////////
@@ -257,6 +324,11 @@ KuduColumnSpec* KuduColumnSpec::Scale(int8_t scale) {
   return this;
 }
 
+KuduColumnSpec* KuduColumnSpec::Length(uint16_t length) {
+  data_->length = length;
+  return this;
+}
+
 KuduColumnSpec* KuduColumnSpec::PrimaryKey() {
   data_->primary_key = true;
   return this;
@@ -303,46 +375,81 @@ Status KuduColumnSpec::ToColumnSchema(KuduColumnSchema* col) const {
     return Status::InvalidArgument("no type provided for column", data_->name);
   }
 
-  if (data_->type.value() == KuduColumnSchema::DECIMAL) {
-    if (!data_->precision) {
-      return Status::InvalidArgument("no precision provided for decimal column", data_->name);
-    }
-    if (data_->precision.value() < kMinDecimalPrecision ||
-        data_->precision.value() > kMaxDecimalPrecision) {
-      return Status::InvalidArgument(
-          strings::Substitute("precision must be between $0 and $1",
-                              kMinDecimalPrecision,
-                              kMaxDecimalPrecision), data_->name);
-    }
-    if (data_->scale) {
-      if (data_->scale.value() < kMinDecimalScale) {
-        return Status::InvalidArgument(
-            strings::Substitute("scale is less than the minimum value of $0",
-                                kMinDecimalScale), data_->name);
+  switch (data_->type.value()) {
+    case KuduColumnSchema::DECIMAL:
+      if (!data_->precision) {
+        return Status::InvalidArgument("no precision provided for decimal column", data_->name);
       }
-      if (data_->scale.value() > data_->precision.value()) {
+      if (data_->precision.value() < kMinDecimalPrecision ||
+          data_->precision.value() > kMaxDecimalPrecision) {
         return Status::InvalidArgument(
-            strings::Substitute("scale is greater than the precision value of",
-                                data_->precision.value()), data_->name);
+            strings::Substitute("precision must be between $0 and $1",
+              kMinDecimalPrecision,
+              kMaxDecimalPrecision), data_->name);
+      }
+      if (data_->scale) {
+        if (data_->scale.value() < kMinDecimalScale) {
+          return Status::InvalidArgument(
+              strings::Substitute("scale is less than the minimum value of $0",
+                kMinDecimalScale), data_->name);
+        }
+        if (data_->scale.value() > data_->precision.value()) {
+          return Status::InvalidArgument(
+              strings::Substitute("scale is greater than the precision value of",
+                data_->precision.value()), data_->name);
+        }
+      }
+      if (data_->length) {
+        return Status::InvalidArgument(
+            strings::Substitute("length is not applicable for column $0",
+              data_->type.value()), data_->name);
+      }
+      break;
+    case KuduColumnSchema::VARCHAR:
+      if (!data_->length) {
+        return Status::InvalidArgument("no length provided for VARCHAR column", data_->name);
+      }
+      if (data_->length.value() < kMinVarcharLength ||
+          data_->length.value() > kMaxVarcharLength) {
+        return Status::InvalidArgument(
+            strings::Substitute("length must be between $0 and $1",
+                                kMinVarcharLength,
+                                kMaxVarcharLength), data_->name);
+      }
+      if (data_->precision) {
+        return Status::InvalidArgument(
+            strings::Substitute("precision is not valid on a $0 column",
+                                data_->type.value()), data_->name);
+      }
+      if (data_->scale) {
+        return Status::InvalidArgument(
+            strings::Substitute("scale is not valid on a $0 column",
+                                data_->type.value()), data_->name);
+      }
+      break;
+    default:
+      if (data_->precision) {
+        return Status::InvalidArgument(
+            strings::Substitute("precision is not valid on a $0 column",
+                                data_->type.value()), data_->name);
+      }
+      if (data_->scale) {
+        return Status::InvalidArgument(
+            strings::Substitute("scale is not valid on a $0 column",
+                                data_->type.value()), data_->name);
+      }
+      if (data_->length) {
+        return Status::InvalidArgument(
+            strings::Substitute("length is not valid on a $0 column",
+                                data_->type.value()), data_->name);
       }
-    }
-  } else {
-    if (data_->precision) {
-      return Status::InvalidArgument(
-          strings::Substitute("precision is not valid on a $0 column",
-                              data_->type.value()), data_->name);
-    }
-    if (data_->scale) {
-      return Status::InvalidArgument(
-          strings::Substitute("scale is not valid on a $0 column",
-                              data_->type.value()), data_->name);
-    }
   }
 
-  int8_t precision = (data_->precision) ? data_->precision.value() : 0;
-  int8_t scale = (data_->scale) ? data_->scale.value() : kDefaultDecimalScale;
+  int8_t precision = data_->precision ? data_->precision.value() : 0;
+  int8_t scale = data_->scale ? data_->scale.value() : kDefaultDecimalScale;
+  uint16_t length = data_->length ? data_->length.value() : 0;
 
-  KuduColumnTypeAttributes type_attrs(precision, scale);
+  KuduColumnTypeAttributes type_attrs(precision, scale, length);
   DataType internal_type = ToInternalDataType(data_->type.value(), type_attrs);
   bool nullable = data_->nullable ? data_->nullable.value() : true;
 
@@ -573,12 +680,54 @@ string KuduColumnSchema::DataTypeToString(DataType type) {
       return "BINARY";
     case UNIXTIME_MICROS:
       return "UNIXTIME_MICROS";
+    case DATE:
+      return "DATE";
     case DECIMAL:
       return "DECIMAL";
+    case VARCHAR:
+      return "VARCHAR";
   }
   LOG(FATAL) << "Unhandled type " << type;
 }
 
+ Status KuduColumnSchema::StringToDataType(
+      const string& type_str, KuduColumnSchema::DataType* type) {
+  Status s;
+  string type_uc;
+  ToUpperCase(type_str, &type_uc);
+  if (type_uc == "INT8") {
+    *type = INT8;
+  } else if (type_uc == "INT16") {
+    *type = INT16;
+  } else if (type_uc == "INT32") {
+    *type = INT32;
+  } else if (type_uc == "INT64") {
+    *type = INT64;
+  } else if (type_uc == "STRING") {
+    *type = STRING;
+  } else if (type_uc == "BOOL") {
+    *type = BOOL;
+  } else if (type_uc == "FLOAT") {
+    *type = FLOAT;
+  } else if (type_uc == "DOUBLE") {
+    *type = DOUBLE;
+  } else if (type_uc == "BINARY") {
+    *type = BINARY;
+  } else if (type_uc == "UNIXTIME_MICROS") {
+    *type = UNIXTIME_MICROS;
+  } else if (type_uc == "DECIMAL") {
+    *type = DECIMAL;
+  } else if (type_uc == "VARCHAR") {
+    *type = VARCHAR;
+  } else if (type_uc == "DATE") {
+    *type = DATE;
+  } else {
+    s = Status::InvalidArgument(Substitute(
+        "data type $0 is not supported", type_str));
+  }
+  return s;
+}
+
 KuduColumnSchema::KuduColumnSchema(const string &name,
                                    DataType type,
                                    bool is_nullable,
@@ -593,6 +742,7 @@ KuduColumnSchema::KuduColumnSchema(const string &name,
   ColumnTypeAttributes type_attr_private;
   type_attr_private.precision = type_attributes.precision();
   type_attr_private.scale = type_attributes.scale();
+  type_attr_private.length = type_attributes.length();
   col_ = new ColumnSchema(name, ToInternalDataType(type, type_attributes),
                           is_nullable,
                           default_value, default_value, attr_private,
@@ -647,7 +797,8 @@ KuduColumnSchema::DataType KuduColumnSchema::type() const {
 
 KuduColumnTypeAttributes KuduColumnSchema::type_attributes() const {
   ColumnTypeAttributes type_attributes = DCHECK_NOTNULL(col_)->type_attributes();
-  return KuduColumnTypeAttributes(type_attributes.precision, type_attributes.scale);
+  return KuduColumnTypeAttributes(type_attributes.precision, type_attributes.scale,
+                                  type_attributes.length);
 }
 
 const string& KuduColumnSchema::comment() const {
@@ -671,6 +822,10 @@ KuduSchema::KuduSchema(const Schema& schema)
   : schema_(new Schema(schema)) {
 }
 
+KuduSchema::KuduSchema(Schema&& schema)
+  : schema_(new Schema(schema)) {
+}
+
 KuduSchema::~KuduSchema() {
   delete schema_;
 }
@@ -689,10 +844,11 @@ void KuduSchema::CopyFrom(const KuduSchema& other) {
 
 Status KuduSchema::Reset(const vector<KuduColumnSchema>& columns, int key_columns) {
   vector<ColumnSchema> cols_private;
-  for (const KuduColumnSchema& col : columns) {
-    cols_private.push_back(*col.col_);
+  cols_private.reserve(columns.size());
+  for (const auto& col : columns) {
+    cols_private.emplace_back(*col.col_);
   }
-  gscoped_ptr<Schema> new_schema(new Schema());
+  unique_ptr<Schema> new_schema(new Schema());
   RETURN_NOT_OK(new_schema->Reset(cols_private, key_columns));
 
   delete schema_;
@@ -706,13 +862,14 @@ bool KuduSchema::Equals(const KuduSchema& other) const {
 }
 
 KuduColumnSchema KuduSchema::Column(size_t idx) const {
-  ColumnSchema col(schema_->column(idx));
+  const ColumnSchema& col = schema_->column(idx);
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
   KuduColumnStorageAttributes attrs(FromInternalEncodingType(col.attributes().encoding),
                                     FromInternalCompressionType(col.attributes().compression));
 #pragma GCC diagnostic pop
-  KuduColumnTypeAttributes type_attrs(col.type_attributes().precision, col.type_attributes().scale);
+  KuduColumnTypeAttributes type_attrs(col.type_attributes().precision, col.type_attributes().scale,
+                                      col.type_attributes().length);
   return KuduColumnSchema(col.name(), FromInternalDataType(col.type_info()->type()),
                           col.is_nullable(), col.read_default_value(),
                           attrs, type_attrs, col.comment());
