@@ -35,8 +35,6 @@
 
 namespace kudu {
 
-class EncodedKey;
-
 namespace client {
 
 namespace internal {
@@ -119,11 +117,6 @@ class KUDU_EXPORT KuduWriteOperation {
   friend class internal::ErrorCollector;
   friend class KuduSession;
 
-  // Create and encode the key for this write (key must be set)
-  //
-  // Caller takes ownership of the allocated memory.
-  EncodedKey* CreateKey() const;
-
   const KuduTable* table() const { return table_.get(); }
 
   // Return the number of bytes required to buffer this operation,
