@@ -18,6 +18,7 @@
 #include "kudu/common/partial_row.h"
 
 #include <cstring>
+#include <ostream>
 #include <string>
 #include <utility>
 
@@ -31,6 +32,7 @@
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/bitmap.h"
+#include "kudu/util/char_util.h"
 #include "kudu/util/decimal_util.h"
 #include "kudu/util/int128.h"
 #include "kudu/util/logging.h"
@@ -44,6 +46,17 @@ using strings::Substitute;
 
 namespace kudu {
 
+namespace {
+Status CheckDateValueInRange(int col_idx, int32_t val, const Schema& schema) {
+  if (PREDICT_FALSE(!DataTypeTraits<DATE>::IsValidValue(val))) {
+    const ColumnSchema& col = schema.column(col_idx);
+    return Status::InvalidArgument(
+        Substitute("value $0 out of range for date column '$1'", val, col.name()));
+  }
+  return Status::OK();
+}
+} // anonymous namespace
+
 KuduPartialRow::KuduPartialRow(const Schema* schema)
   : schema_(schema) {
   DCHECK(schema_->initialized());
@@ -62,7 +75,7 @@ KuduPartialRow::KuduPartialRow(const Schema* schema)
                        row_size, "NEWNEWNEWNEWNEW");
 #endif
   ContiguousRowHelper::InitNullsBitmap(
-    *schema_, row_data_, ContiguousRowHelper::null_bitmap_size(*schema_));
+    *schema_, row_data_, ContiguousRowHelper::non_null_bitmap_size(*schema_));
 }
 
 KuduPartialRow::~KuduPartialRow() {
@@ -188,10 +201,18 @@ Status KuduPartialRow::Set(int32_t column_idx, const uint8_t* val) {
       RETURN_NOT_OK(SetBinaryCopy(column_idx, *reinterpret_cast<const Slice*>(val)));
       break;
     }
+    case VARCHAR: {
+      RETURN_NOT_OK(SetVarchar(column_idx, *reinterpret_cast<const Slice*>(val)));
+      break;
+    }
     case UNIXTIME_MICROS: {
       RETURN_NOT_OK(SetUnixTimeMicros(column_idx, *reinterpret_cast<const int64_t*>(val)));
       break;
     }
+    case DATE: {
+      RETURN_NOT_OK(SetDate(column_idx, *reinterpret_cast<const int32_t*>(val)));
+      break;
+    }
     case DECIMAL32: {
       RETURN_NOT_OK(Set<TypeTraits<DECIMAL32> >(column_idx,
                                                 *reinterpret_cast<const int32_t*>(val)));
@@ -218,11 +239,19 @@ void KuduPartialRow::DeallocateStringIfSet(int col_idx, const ColumnSchema& col)
   if (BitmapTest(owned_strings_bitmap_, col_idx)) {
     ContiguousRow row(schema_, row_data_);
     const Slice* dst;
-    if (col.type_info()->type() == BINARY) {
-      dst = schema_->ExtractColumnFromRow<BINARY>(row, col_idx);
-    } else {
-      CHECK(col.type_info()->type() == STRING);
-      dst = schema_->ExtractColumnFromRow<STRING>(row, col_idx);
+    switch (col.type_info()->type()) {
+      case BINARY:
+        dst = schema_->ExtractColumnFromRow<BINARY>(row, col_idx);
+        break;
+      case VARCHAR:
+        dst = schema_->ExtractColumnFromRow<VARCHAR>(row, col_idx);
+        break;
+      case STRING:
+        dst = schema_->ExtractColumnFromRow<STRING>(row, col_idx);
+        break;
+      default:
+        LOG(FATAL) << "Unexpected type " << col.type_info()->type();
+        break;
     }
     delete [] dst->data();
     BitmapClear(owned_strings_bitmap_, col_idx);
@@ -254,8 +283,11 @@ Status KuduPartialRow::SetInt32(const Slice& col_name, int32_t val) {
 Status KuduPartialRow::SetInt64(const Slice& col_name, int64_t val) {
   return Set<TypeTraits<INT64> >(col_name, val);
 }
-Status KuduPartialRow::SetUnixTimeMicros(const Slice& col_name, int64_t val) {
-  return Set<TypeTraits<UNIXTIME_MICROS> >(col_name, val);
+Status KuduPartialRow::SetUnixTimeMicros(const Slice& col_name, int64_t micros_since_utc_epoch) {
+  return Set<TypeTraits<UNIXTIME_MICROS> >(col_name, micros_since_utc_epoch);
+}
+Status KuduPartialRow::SetDate(const Slice& col_name, int32_t days_since_unix_epoch) {
+  return Set<TypeTraits<DATE> >(col_name, days_since_unix_epoch);
 }
 Status KuduPartialRow::SetFloat(const Slice& col_name, float val) {
   return Set<TypeTraits<FLOAT> >(col_name, val);
@@ -283,8 +315,12 @@ Status KuduPartialRow::SetInt32(int col_idx, int32_t val) {
 Status KuduPartialRow::SetInt64(int col_idx, int64_t val) {
   return Set<TypeTraits<INT64> >(col_idx, val);
 }
-Status KuduPartialRow::SetUnixTimeMicros(int col_idx, int64_t val) {
-  return Set<TypeTraits<UNIXTIME_MICROS> >(col_idx, val);
+Status KuduPartialRow::SetUnixTimeMicros(int col_idx, int64_t micros_since_utc_epoch) {
+  return Set<TypeTraits<UNIXTIME_MICROS> >(col_idx, micros_since_utc_epoch);
+}
+Status KuduPartialRow::SetDate(int col_idx, int32_t days_since_unix_epoch) {
+  RETURN_NOT_OK(CheckDateValueInRange(col_idx, days_since_unix_epoch, *schema_));
+  return Set<TypeTraits<DATE> >(col_idx, days_since_unix_epoch);
 }
 Status KuduPartialRow::SetFloat(int col_idx, float val) {
   return Set<TypeTraits<FLOAT> >(col_idx, val);
@@ -330,12 +366,21 @@ Status KuduPartialRow::SetBinary(const Slice& col_name, const Slice& val) {
 Status KuduPartialRow::SetString(const Slice& col_name, const Slice& val) {
   return SetStringCopy(col_name, val);
 }
+Status KuduPartialRow::SetVarchar(const Slice& col_name, const Slice& val) {
+  int col_idx;
+  RETURN_NOT_OK(schema_->FindColumn(col_name, &col_idx));
+  return SetVarchar(col_idx, val);
+}
+
 Status KuduPartialRow::SetBinary(int col_idx, const Slice& val) {
   return SetBinaryCopy(col_idx, val);
 }
 Status KuduPartialRow::SetString(int col_idx, const Slice& val) {
   return SetStringCopy(col_idx, val);
 }
+Status KuduPartialRow::SetVarchar(int col_idx, const Slice& val) {
+  return SetSliceCopy<TypeTraits<VARCHAR> >(col_idx, val);
+}
 
 Status KuduPartialRow::SetBinaryCopy(const Slice& col_name, const Slice& val) {
   return SetSliceCopy<TypeTraits<BINARY> >(col_name, val);
@@ -363,26 +408,47 @@ Status KuduPartialRow::SetStringNoCopy(int col_idx, const Slice& val) {
   return Set<TypeTraits<STRING> >(col_idx, val, false);
 }
 
+Status KuduPartialRow::SetVarcharNoCopyUnsafe(const Slice& col_name, const Slice& val) {
+  int col_idx;
+  RETURN_NOT_OK(schema_->FindColumn(col_name, &col_idx));
+  return SetVarcharNoCopyUnsafe(col_idx, val);
+}
+
+Status KuduPartialRow::SetVarcharNoCopyUnsafe(int col_idx, const Slice& val) {
+  const auto& col = schema_->column(col_idx);
+  if (val.size() > col.type_attributes().length * 4) {
+    return Status::InvalidArgument(
+        Substitute("Value too long, limit is $0 characters",
+                   col.type_attributes().length));
+  }
+  return Set<TypeTraits<VARCHAR> >(col_idx, val);
+}
+
 template<typename T>
 Status KuduPartialRow::SetSliceCopy(const Slice& col_name, const Slice& val) {
-  auto relocated = new uint8_t[val.size()];
-  memcpy(relocated, val.data(), val.size());
-  Slice relocated_val(relocated, val.size());
-  Status s = Set<T>(col_name, relocated_val, true);
-  if (!s.ok()) {
-    delete [] relocated;
-  }
-  return s;
+  int col_idx;
+  RETURN_NOT_OK(schema_->FindColumn(col_name, &col_idx));
+  return SetSliceCopy<T>(col_idx, val);
 }
 
 template<typename T>
 Status KuduPartialRow::SetSliceCopy(int col_idx, const Slice& val) {
-  auto relocated = new uint8_t[val.size()];
-  memcpy(relocated, val.data(), val.size());
-  Slice relocated_val(relocated, val.size());
+  const auto& col = schema_->column(col_idx);
+  Slice relocated_val;
+  switch (T::type) {
+    case VARCHAR:
+      relocated_val = UTF8Truncate(val, col.type_attributes().length);
+      break;
+    case STRING:
+    case BINARY:
+      auto relocated = new uint8_t[val.size()];
+      memcpy(relocated, val.data(), val.size());
+      relocated_val = Slice(relocated, val.size());
+      break;
+  }
   Status s = Set<T>(col_idx, relocated_val, true);
   if (!s.ok()) {
-    delete [] relocated;
+    delete [] relocated_val.data();
   }
   return s;
 }
@@ -471,6 +537,11 @@ Status KuduPartialRow::Set<TypeTraits<UNIXTIME_MICROS> >(
     const TypeTraits<UNIXTIME_MICROS>::cpp_type& val,
     bool owned);
 
+template
+Status KuduPartialRow::Set<TypeTraits<DATE> >(int col_idx,
+                                              const TypeTraits<DATE>::cpp_type& val,
+                                              bool owned);
+
 template
 Status KuduPartialRow::Set<TypeTraits<STRING> >(int col_idx,
                                                 const TypeTraits<STRING>::cpp_type& val,
@@ -543,6 +614,11 @@ Status KuduPartialRow::Set<TypeTraits<UNIXTIME_MICROS> >(
     const TypeTraits<UNIXTIME_MICROS>::cpp_type& val,
     bool owned);
 
+template
+Status KuduPartialRow::Set<TypeTraits<DATE> >(const Slice& col_name,
+                                              const TypeTraits<DATE>::cpp_type& val,
+                                              bool owned);
+
 template
 Status KuduPartialRow::Set<TypeTraits<FLOAT> >(const Slice& col_name,
                                                const TypeTraits<FLOAT>::cpp_type& val,
@@ -635,6 +711,10 @@ Status KuduPartialRow::GetUnixTimeMicros(const Slice& col_name,
                                          int64_t* micros_since_utc_epoch) const {
   return Get<TypeTraits<UNIXTIME_MICROS> >(col_name, micros_since_utc_epoch);
 }
+Status KuduPartialRow::GetDate(const Slice& col_name,
+                               int32_t* days_since_unix_epoch) const {
+  return Get<TypeTraits<DATE> >(col_name, days_since_unix_epoch);
+}
 Status KuduPartialRow::GetFloat(const Slice& col_name, float* val) const {
   return Get<TypeTraits<FLOAT> >(col_name, val);
 }
@@ -656,6 +736,11 @@ Status KuduPartialRow::GetString(const Slice& col_name, Slice* val) const {
 Status KuduPartialRow::GetBinary(const Slice& col_name, Slice* val) const {
   return Get<TypeTraits<BINARY> >(col_name, val);
 }
+Status KuduPartialRow::GetVarchar(const Slice& col_name, Slice* val) const {
+  int col_idx;
+  RETURN_NOT_OK(schema_->FindColumn(col_name, &col_idx));
+  return GetVarchar(col_idx, val);
+}
 
 Status KuduPartialRow::GetBool(int col_idx, bool* val) const {
   return Get<TypeTraits<BOOL> >(col_idx, val);
@@ -675,6 +760,9 @@ Status KuduPartialRow::GetInt64(int col_idx, int64_t* val) const {
 Status KuduPartialRow::GetUnixTimeMicros(int col_idx, int64_t* micros_since_utc_epoch) const {
   return Get<TypeTraits<UNIXTIME_MICROS> >(col_idx, micros_since_utc_epoch);
 }
+Status KuduPartialRow::GetDate(int col_idx, int32_t* days_since_unix_epoch) const {
+  return Get<TypeTraits<DATE> >(col_idx, days_since_unix_epoch);
+}
 Status KuduPartialRow::GetFloat(int col_idx, float* val) const {
   return Get<TypeTraits<FLOAT> >(col_idx, val);
 }
@@ -716,6 +804,9 @@ Status KuduPartialRow::GetString(int col_idx, Slice* val) const {
 Status KuduPartialRow::GetBinary(int col_idx, Slice* val) const {
   return Get<TypeTraits<BINARY> >(col_idx, val);
 }
+Status KuduPartialRow::GetVarchar(int col_idx, Slice* val) const {
+  return Get<TypeTraits<VARCHAR> >(col_idx, val);
+}
 
 template<typename T>
 Status KuduPartialRow::Get(const Slice& col_name,
