@@ -19,7 +19,6 @@
 
 #include <algorithm>
 #include <cstring>
-#include <memory>
 #include <set>
 #include <string>
 #include <unordered_set>
@@ -27,6 +26,7 @@
 #include <vector>
 
 #include <glog/logging.h>
+#include <google/protobuf/stubs/port.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/key_encoder.h"
@@ -1013,6 +1013,12 @@ namespace {
       case UNIXTIME_MICROS:
         RETURN_NOT_OK(row->SetInt64(idx, INT64_MIN + 1));
         break;
+      case DATE:
+        RETURN_NOT_OK(row->SetDate(idx, DataTypeTraits<DATE>::kMinValue + 1));
+        break;
+      case VARCHAR:
+        RETURN_NOT_OK(row->SetVarchar(idx, Slice("\0", 1)));
+        break;
       case STRING:
         RETURN_NOT_OK(row->SetStringCopy(idx, Slice("\0", 1)));
         break;
@@ -1089,6 +1095,16 @@ namespace {
         }
         break;
       }
+      case DATE: {
+        int32_t value;
+        RETURN_NOT_OK(row->GetDate(idx, &value));
+        if (value < DataTypeTraits<DATE>::kMaxValue) {
+          RETURN_NOT_OK(row->SetDate(idx, value + 1));
+        } else {
+          *success = false;
+        }
+        break;
+      }
       case DECIMAL32:
       case DECIMAL64:
       case DECIMAL128: {
@@ -1109,6 +1125,14 @@ namespace {
         RETURN_NOT_OK(row->SetBinaryCopy(idx, incremented));
         break;
       }
+      case VARCHAR: {
+        Slice value;
+        RETURN_NOT_OK(row->GetVarchar(idx, &value));
+        string incremented = value.ToString();
+        incremented.push_back('\0');
+        RETURN_NOT_OK(row->SetVarchar(idx, incremented));
+        break;
+      }
       case STRING: {
         Slice value;
         RETURN_NOT_OK(row->GetString(idx, &value));
