@@ -23,13 +23,14 @@
 #include <memory>
 #include <ostream>
 #include <string>
-#include <unordered_map>
+#include <type_traits>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
 #include <boost/optional/optional.hpp>
 #include <glog/logging.h>
+#include <sparsehash/dense_hash_map>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/id_mapping.h"
@@ -37,7 +38,6 @@
 #include "kudu/common/types.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/port.h"
-#include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/strcat.h"
 #include "kudu/gutil/strings/stringpiece.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -46,6 +46,10 @@
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 
+namespace kudu {
+class Schema;
+}  // namespace kudu
+
 // Check that two schemas are equal, yielding a useful error message in the case that
 // they are not.
 #define DCHECK_SCHEMA_EQ(s1, s2) \
@@ -61,6 +65,8 @@
                                  << (s2).ToString(); \
   } while (0)
 
+template <class X> struct GoodFastHash;
+
 namespace kudu {
 
 class Arena;
@@ -92,12 +98,20 @@ struct ColumnTypeAttributes {
  public:
   ColumnTypeAttributes()
       : precision(0),
-        scale(0) {
+        scale(0),
+        length(0) {
   }
 
   ColumnTypeAttributes(int8_t precision, int8_t scale)
       : precision(precision),
-        scale(scale) {
+        scale(scale),
+        length(0) {
+  }
+
+  explicit ColumnTypeAttributes(uint16_t length)
+      : precision(0),
+        scale(0),
+        length(length) {
   }
 
   // Does `other` represent equivalent attributes for `type`?
@@ -112,6 +126,12 @@ struct ColumnTypeAttributes {
 
   int8_t precision;
   int8_t scale;
+
+  // Maximum value of the length is 65,535 for compatibility reasons as it's
+  // used by VARCHAR type which can be set to a maximum of 65,535 in case of
+  // MySQL and less for other major RDBMS implementations. The length refers to
+  // the number of characters/symbols (not bytes).
+  uint16_t length;
 };
 
 // Class for storing column attributes such as compression and
@@ -210,14 +230,14 @@ class ColumnSchema {
       : name_(std::move(name)),
         type_info_(GetTypeInfo(type)),
         is_nullable_(is_nullable),
-        read_default_(read_default ? new Variant(type, read_default) : nullptr),
+        read_default_(read_default ? std::make_shared<Variant>(type, read_default) : nullptr),
         attributes_(attributes),
         type_attributes_(type_attributes),
         comment_(std::move(comment)) {
     if (write_default == read_default) {
       write_default_ = read_default_;
     } else if (write_default != nullptr) {
-      write_default_.reset(new Variant(type, write_default));
+      write_default_ = std::make_shared<Variant>(type, write_default);
     }
   }
 
@@ -438,16 +458,12 @@ class Schema {
 
   Schema()
     : num_key_columns_(0),
-      name_to_index_bytes_(0),
-      // TODO(wdberkeley): C++11 provides a single-argument constructor, but
-      // it's not supported in GCC < 4.9. This (and the other occurrences here
-      // and in schema.cc) can be fixed if we adopt C++14 or a later standard.
-      name_to_index_(/*bucket_count=*/10,
-                     NameToIndexMap::hasher(),
-                     NameToIndexMap::key_equal(),
-                     NameToIndexMapAllocator(&name_to_index_bytes_)),
+      // Initialize for 1 expected element since 0 gets replaced with
+      // the default (32).
+      name_to_index_(1),
       first_is_deleted_virtual_column_idx_(kColumnNotFound),
       has_nullables_(false) {
+    name_to_index_.set_empty_key(StringPiece());
   }
 
   Schema(const Schema& other);
@@ -464,14 +480,11 @@ class Schema {
   // empty schema and then use Reset(...)  so that errors can be
   // caught. If an invalid schema is passed to this constructor, an
   // assertion will be fired!
-  Schema(const std::vector<ColumnSchema>& cols,
+  Schema(std::vector<ColumnSchema> cols,
          int key_columns)
-    : name_to_index_bytes_(0),
-      name_to_index_(/*bucket_count=*/10,
-                     NameToIndexMap::hasher(),
-                     NameToIndexMap::key_equal(),
-                     NameToIndexMapAllocator(&name_to_index_bytes_)) {
-    CHECK_OK(Reset(cols, key_columns));
+      : name_to_index_(/*expected_max_items_in_table=*/cols.size()) {
+    name_to_index_.set_empty_key(StringPiece());
+    CHECK_OK(Reset(std::move(cols), key_columns));
   }
 
   // Construct a schema with the given information.
@@ -480,30 +493,27 @@ class Schema {
   // empty schema and then use Reset(...)  so that errors can be
   // caught. If an invalid schema is passed to this constructor, an
   // assertion will be fired!
-  Schema(const std::vector<ColumnSchema>& cols,
-         const std::vector<ColumnId>& ids,
+  Schema(std::vector<ColumnSchema> cols,
+         std::vector<ColumnId> ids,
          int key_columns)
-    : name_to_index_bytes_(0),
-      name_to_index_(/*bucket_count=*/10,
-                     NameToIndexMap::hasher(),
-                     NameToIndexMap::key_equal(),
-                     NameToIndexMapAllocator(&name_to_index_bytes_)) {
-    CHECK_OK(Reset(cols, ids, key_columns));
+      : name_to_index_(/*expected_max_items_in_table=*/cols.size()) {
+    name_to_index_.set_empty_key(StringPiece());
+    CHECK_OK(Reset(std::move(cols), std::move(ids), key_columns));
   }
 
   // Reset this Schema object to the given schema.
   // If this fails, the Schema object is left in an inconsistent
   // state and may not be used.
-  Status Reset(const std::vector<ColumnSchema>& cols, int key_columns) {
+  Status Reset(std::vector<ColumnSchema> cols, int key_columns) {
     std::vector<ColumnId> ids;
-    return Reset(cols, ids, key_columns);
+    return Reset(std::move(cols), ids, key_columns);
   }
 
   // Reset this Schema object to the given schema.
   // If this fails, the Schema object is left in an inconsistent
   // state and may not be used.
-  Status Reset(const std::vector<ColumnSchema>& cols,
-               const std::vector<ColumnId>& ids,
+  Status Reset(std::vector<ColumnSchema> cols,
+               std::vector<ColumnId> ids,
                int key_columns);
 
   // Find the column index corresponding to the given column name,
@@ -718,7 +728,7 @@ class Schema {
       col_ids.assign(col_ids_.begin(), col_ids_.begin() + num_key_columns_);
     }
 
-    return Schema(key_cols, col_ids, num_key_columns_);
+    return Schema(std::move(key_cols), std::move(col_ids), num_key_columns_);
   }
 
   // Return a new Schema which is the same as this one, but with IDs assigned.
@@ -959,17 +969,12 @@ class Schema {
   // ColumnSchema objects inside cols_. This avoids an extra copy of those strings,
   // and also allows us to do lookups on the map using StringPiece keys, sometimes
   // avoiding copies.
-  //
-  // The map is instrumented with a counting allocator so that we can accurately
-  // measure its memory footprint.
-  int64_t name_to_index_bytes_;
-  typedef STLCountingAllocator<std::pair<const StringPiece, size_t> > NameToIndexMapAllocator;
-  typedef std::unordered_map<
+  typedef google::dense_hash_map<
       StringPiece,
       size_t,
-      std::hash<StringPiece>,
-      std::equal_to<StringPiece>,
-      NameToIndexMapAllocator> NameToIndexMap;
+      GoodFastHash<StringPiece>,
+    std::equal_to<StringPiece>> NameToIndexMap;
+
   NameToIndexMap name_to_index_;
 
   IdMapping id_to_index_;
