@@ -384,7 +384,7 @@ Status PeerMessageQueue::AppendOperation(const ReplicateRefPtr& msg) {
   });
 }
 
-Status PeerMessageQueue::AppendOperations(const vector<ReplicateRefPtr>& msgs,
+Status PeerMessageQueue::AppendOperations(vector<ReplicateRefPtr> msgs,
                                           const StatusCallback& log_append_callback) {
 
   DFAKE_SCOPED_LOCK(append_fake_lock_);
@@ -424,7 +424,7 @@ Status PeerMessageQueue::AppendOperations(const vector<ReplicateRefPtr>& msgs,
   // which also needs queue_lock_.
   lock.unlock();
   RETURN_NOT_OK(log_cache_.AppendOperations(
-      msgs, [this, last_id, log_append_callback](const Status& s) {
+      std::move(msgs), [this, last_id, log_append_callback](const Status& s) {
         this->LocalPeerAppendFinished(last_id, log_append_callback, s);
       }));
   lock.lock();
@@ -693,7 +693,7 @@ Status PeerMessageQueue::RequestForPeer(const string& uuid,
 
     // The batch of messages to send to the peer.
     vector<ReplicateRefPtr> messages;
-    int max_batch_size = FLAGS_consensus_max_batch_size_bytes - request->ByteSize();
+    int64_t max_batch_size = FLAGS_consensus_max_batch_size_bytes - request->ByteSizeLong();
 
     // We try to get the follower's next_index from our log.
     Status s = log_cache_.ReadOps(peer_copy.next_index - 1,
