@@ -101,9 +101,9 @@ enum class PeerStatus {
 
 const char* PeerStatusToString(PeerStatus p);
 
-// Tracks the state of the peers and which transactions they have replicated.
-// Owns the LogCache which actually holds the replicate messages which are
-// en route to the various peers.
+// Tracks the state of the peers and which ops they have replicated. Owns the
+// LogCache which actually holds the replicate messages which are en route to
+// the various peers.
 //
 // This also takes care of pushing requests to peers as new operations are
 // added, and notifying RaftConsensus when the commit index advances.
@@ -241,7 +241,7 @@ class PeerMessageQueue {
   //
   // This is thread-safe against all of the read methods, but not thread-safe
   // with concurrent Append calls.
-  Status AppendOperations(const std::vector<ReplicateRefPtr>& msgs,
+  Status AppendOperations(std::vector<ReplicateRefPtr> msgs,
                           const StatusCallback& log_append_callback);
 
   // Truncate all operations coming after 'index'. Following this, the 'last_appended'
