@@ -15,25 +15,27 @@
 // specific language governing permissions and limitations
 // under the License.
 
+#include "kudu/fs/block_manager.h"
+
+#include <stdlib.h>
+
 #include <algorithm>
 #include <cstddef>
 #include <cstdint>
+#include <initializer_list>
 #include <memory>
 #include <ostream>
-#include <stdlib.h>
 #include <string>
 #include <thread>
 #include <unordered_set>
 #include <vector>
 
 #include <gflags/gflags.h>
-#include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <google/protobuf/util/message_differencer.h>
 #include <gtest/gtest.h>
 
 #include "kudu/fs/block_id.h"
-#include "kudu/fs/block_manager.h"
 #include "kudu/fs/data_dirs.h"
 #include "kudu/fs/error_manager.h"
 #include "kudu/fs/file_block_manager.h"
@@ -41,14 +43,13 @@
 #include "kudu/fs/fs_report.h"
 #include "kudu/fs/log_block_manager.h"
 #include "kudu/gutil/basictypes.h"
-#include "kudu/gutil/bind.h"
 #include "kudu/gutil/casts.h"
-#include "kudu/gutil/gscoped_ptr.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/array_view.h" // IWYU pragma: keep
 #include "kudu/util/env.h"
+#include "kudu/util/file_cache.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
@@ -60,11 +61,11 @@
 #include "kudu/util/stopwatch.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
-#include "kudu/util/thread.h"
 
 using google::protobuf::util::MessageDifferencer;
 using std::shared_ptr;
 using std::string;
+using std::thread;
 using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
@@ -105,15 +106,25 @@ namespace fs {
 
 static const char* kTestData = "test data";
 
+template<typename T>
+string block_manager_type();
+
+template<>
+string block_manager_type<FileBlockManager>() { return "file"; }
+
+template<>
+string block_manager_type<LogBlockManager>() { return "log"; }
+
 template <typename T>
 class BlockManagerTest : public KuduTest {
  public:
   BlockManagerTest() :
-    test_tablet_name_("test_tablet"),
-    test_block_opts_(CreateBlockOptions({ test_tablet_name_ })),
-    test_error_manager_(new FsErrorManager()),
-    bm_(CreateBlockManager(scoped_refptr<MetricEntity>(),
-                           shared_ptr<MemTracker>())) {
+      test_tablet_name_("test_tablet"),
+      test_block_opts_(CreateBlockOptions({ test_tablet_name_ })),
+      file_cache_("test_cache", env_, 1, scoped_refptr<MetricEntity>()),
+      bm_(CreateBlockManager(scoped_refptr<MetricEntity>(),
+                             shared_ptr<MemTracker>())) {
+    CHECK_OK(file_cache_.Init());
   }
 
   virtual void SetUp() override {
@@ -144,18 +155,21 @@ class BlockManagerTest : public KuduTest {
   }
 
  protected:
+
   T* CreateBlockManager(const scoped_refptr<MetricEntity>& metric_entity,
                         const shared_ptr<MemTracker>& parent_mem_tracker) {
     if (!dd_manager_) {
+      DataDirManagerOptions opts;
+      opts.dir_type = block_manager_type<T>();
       // Create a new directory manager if necessary.
       CHECK_OK(DataDirManager::CreateNewForTests(env_, { test_dir_ },
-          DataDirManagerOptions(), &dd_manager_));
+          opts, &dd_manager_));
     }
     BlockManagerOptions opts;
     opts.metric_entity = metric_entity;
     opts.parent_mem_tracker = parent_mem_tracker;
-    return new T(env_, this->dd_manager_.get(), test_error_manager_.get(),
-                 std::move(opts));
+    return new T(env_, this->dd_manager_.get(), &error_manager_,
+                 &file_cache_, std::move(opts));
   }
 
   Status ReopenBlockManager(const scoped_refptr<MetricEntity>& metric_entity,
@@ -167,13 +181,14 @@ class BlockManagerTest : public KuduTest {
     // manager first to enforce this.
     bm_.reset();
     DataDirManagerOptions opts;
+    opts.dir_type = block_manager_type<T>();
     opts.metric_entity = metric_entity;
     if (create) {
       RETURN_NOT_OK(DataDirManager::CreateNewForTests(
-          env_, paths, std::move(opts), &dd_manager_));
+          env_, paths, opts, &dd_manager_));
     } else {
       RETURN_NOT_OK(DataDirManager::OpenExistingForTests(
-          env_, paths, std::move(opts), &dd_manager_));
+          env_, paths, opts, &dd_manager_));
     }
     bm_.reset(CreateBlockManager(metric_entity, parent_mem_tracker));
     RETURN_NOT_OK(bm_->Open(nullptr));
@@ -208,17 +223,20 @@ class BlockManagerTest : public KuduTest {
   // hierarchy, ignoring '.', '..', and file 'kInstanceMetadataFileName'.
   Status CountFiles(const string& root, int* num_files) {
     *num_files = 0;
-    RETURN_NOT_OK(env_->Walk(root, Env::PRE_ORDER,
-                             Bind(BlockManagerTest::CountFilesCb, num_files)));
-    return Status::OK();
+    return env_->Walk(
+        root, Env::PRE_ORDER,
+        [num_files](Env::FileType type, const string& dirname, const string& basename) {
+          return CountFilesCb(num_files, type, dirname, basename);
+        });
   }
 
   // Keep an internal copy of the data dir group to act as metadata.
   DataDirGroupPB test_group_pb_;
   string test_tablet_name_;
   CreateBlockOptions test_block_opts_;
-  unique_ptr<FsErrorManager> test_error_manager_;
+  FsErrorManager error_manager_;
   unique_ptr<DataDirManager> dd_manager_;
+  FileCache file_cache_;
   unique_ptr<T> bm_;
 };
 
@@ -299,7 +317,7 @@ void BlockManagerTest<LogBlockManager>::RunBlockDistributionTest(const vector<st
 template <>
 void BlockManagerTest<FileBlockManager>::RunMultipathTest(const vector<string>& paths) {
   // Ensure that each path has an instance file and that it's well-formed.
-  for (const string& path : dd_manager_->GetDataDirs()) {
+  for (const string& path : dd_manager_->GetDirs()) {
     vector<string> children;
     ASSERT_OK(env_->GetChildren(path, &children));
     ASSERT_EQ(3, children.size());
@@ -307,7 +325,7 @@ void BlockManagerTest<FileBlockManager>::RunMultipathTest(const vector<string>&
       if (child == "." || child == "..") {
         continue;
       }
-      PathInstanceMetadataPB instance;
+      DirInstanceMetadataPB instance;
       ASSERT_OK(pb_util::ReadPBContainerFromPath(env_,
                                                  JoinPathSegments(path, child),
                                                  &instance));
@@ -421,6 +439,50 @@ typedef ::testing::Types<FileBlockManager> BlockManagers;
 #endif
 TYPED_TEST_CASE(BlockManagerTest, BlockManagers);
 
+// Test to make sure that we don't break the file block manager, which depends
+// on a static set of directories to function properly. Internally, the
+// DataDirManager of a file block manager must use a specific ordering for its
+// directory UUID indexes which is persisted with the PIMFs.
+TYPED_TEST(BlockManagerTest, TestOpenWithDifferentDirOrder) {
+  const string path1 = this->GetTestPath("path1");
+  const string path2 = this->GetTestPath("path2");
+  vector<string> paths = { path1, path2 };
+  ASSERT_OK(this->env_->CreateDir(path1));
+  ASSERT_OK(this->env_->CreateDir(path2));
+  ASSERT_OK(this->ReopenBlockManager(scoped_refptr<MetricEntity>(),
+                                     shared_ptr<MemTracker>(),
+                                     paths,
+                                     true /* create */,
+                                     false /* load_test_group */));
+
+  const string kTablet = "tablet";
+  CreateBlockOptions opts({ kTablet });
+  FLAGS_fs_target_data_dirs_per_tablet = 2;
+  ASSERT_OK(this->dd_manager_->CreateDataDirGroup(kTablet));
+
+  // Create a block and keep track of its block id.
+  unique_ptr<BlockCreationTransaction> transaction = this->bm_->NewCreationTransaction();
+  unique_ptr<WritableBlock> written_block;
+  ASSERT_OK(this->bm_->CreateBlock(opts, &written_block));
+  const auto block_id = written_block->id();
+  ASSERT_OK(written_block->Append(kTestData));
+  transaction->AddCreatedBlock(std::move(written_block));
+  ASSERT_OK(transaction->CommitCreatedBlocks());
+
+  // Now reopen the block manager with a different ordering for the data
+  // directories.
+  paths = { path2, path1 };
+  ASSERT_OK(this->ReopenBlockManager(scoped_refptr<MetricEntity>(),
+                                     shared_ptr<MemTracker>(),
+                                     paths,
+                                     false /* create */,
+                                     false /* load_test_group */));
+
+  // We should have no trouble reading the block back.
+  unique_ptr<ReadableBlock> read_block;
+  ASSERT_OK(this->bm_->OpenBlock(block_id, &read_block));
+}
+
 // Test the entire lifecycle of a block.
 TYPED_TEST(BlockManagerTest, EndToEndTest) {
   // Create a block.
@@ -514,7 +576,7 @@ TYPED_TEST(BlockManagerTest, ReadAfterDeleteTest) {
               .IsNotFound());
 
   // But we should still be able to read from the opened block.
-  gscoped_ptr<uint8_t[]> scratch(new uint8_t[test_data.length()]);
+  unique_ptr<uint8_t[]> scratch(new uint8_t[test_data.length()]);
   Slice data(scratch.get(), test_data.length());
   ASSERT_OK(read_block->Read(0, data));
   ASSERT_EQ(test_data, data);
@@ -683,7 +745,7 @@ TYPED_TEST(BlockManagerTest, PersistenceTest) {
   // The existing block manager is left open, which proxies for the process
   // having crashed without cleanly shutting down the block manager. The
   // on-disk metadata should still be clean.
-  gscoped_ptr<BlockManager> new_bm(this->CreateBlockManager(
+  unique_ptr<BlockManager> new_bm(this->CreateBlockManager(
       scoped_refptr<MetricEntity>(),
       MemTracker::CreateTracker(-1, "other tracker")));
   ASSERT_OK(new_bm->Open(nullptr));
@@ -698,7 +760,7 @@ TYPED_TEST(BlockManagerTest, PersistenceTest) {
   ASSERT_OK(new_bm->OpenBlock(written_block2->id(), &read_block));
   ASSERT_OK(read_block->Size(&sz));
   ASSERT_EQ(test_data.length(), sz);
-  gscoped_ptr<uint8_t[]> scratch(new uint8_t[test_data.length()]);
+  unique_ptr<uint8_t[]> scratch(new uint8_t[test_data.length()]);
   Slice data(scratch.get(), test_data.length());
   ASSERT_OK(read_block->Read(0, data));
   ASSERT_EQ(test_data, data);
@@ -750,15 +812,14 @@ TYPED_TEST(BlockManagerTest, ConcurrentCloseReadableBlockTest) {
   unique_ptr<ReadableBlock> reader;
   ASSERT_OK(this->bm_->OpenBlock(writer->id(), &reader));
 
-  vector<scoped_refptr<Thread> > threads;
-  for (int i = 0; i < 100; i++) {
-    scoped_refptr<Thread> t;
-    ASSERT_OK(Thread::Create("test", Substitute("t$0", i),
-                             &CloseHelper, reader.get(), &t));
-    threads.push_back(t);
+  constexpr int kNumThreads = 100;
+  vector<thread> threads;
+  threads.reserve(kNumThreads);
+  for (int i = 0; i < kNumThreads; i++) {
+    threads.emplace_back([&reader]() { CloseHelper(reader.get()); });
   }
-  for (const scoped_refptr<Thread>& t : threads) {
-    t->Join();
+  for (auto& t : threads) {
+    t.join();
   }
 }
 
@@ -797,7 +858,7 @@ TYPED_TEST(BlockManagerTest, MetricsTest) {
         i * kTestData.length(), (i + 1) * kTestData.length()));
 
     // The read is reflected in total_bytes_read.
-    gscoped_ptr<uint8_t[]> scratch(new uint8_t[kTestData.length()]);
+    unique_ptr<uint8_t[]> scratch(new uint8_t[kTestData.length()]);
     Slice data(scratch.get(), kTestData.length());
     ASSERT_OK(reader->Read(0, data));
     NO_FATALS(CheckMetrics(
@@ -880,7 +941,7 @@ TYPED_TEST(BlockManagerTest, TestMetadataOkayDespiteFailure) {
 
   // Creates a block with the given 'test_data', writing the result
   // to 'out' on success.
-  auto create_a_block = [&](BlockId* out, const string& test_data) -> Status {
+  auto create_a_block = [&](BlockId* out, const string& test_data) {
     unique_ptr<WritableBlock> block;
     RETURN_NOT_OK(this->bm_->CreateBlock(this->test_block_opts_, &block));
     for (int i = 0; i < kNumAppends; i++) {
@@ -896,7 +957,7 @@ TYPED_TEST(BlockManagerTest, TestMetadataOkayDespiteFailure) {
   // Reads a block given by 'id', comparing its contents. Note that
   // we need to compare with both kLongTestData and kShortTestData as we
   // do not know the blocks' content ahead.
-  auto read_a_block = [&](const BlockId& id) -> Status {
+  auto read_a_block = [&](const BlockId& id) {
     unique_ptr<ReadableBlock> block;
     RETURN_NOT_OK(this->bm_->OpenBlock(id, &block));
     uint64_t size;
@@ -1036,8 +1097,10 @@ TYPED_TEST(BlockManagerTest, ConcurrentCloseFinalizedWritableBlockTest) {
     }
   };
 
-  vector<std::thread> threads;
-  for (int i = 0; i < 100; i++) {
+  constexpr int kNumThreads = 100;
+  vector<thread> threads;
+  threads.reserve(kNumThreads);
+  for (int i = 0; i < kNumThreads; i++) {
     threads.emplace_back(write_data);
   }
   for (auto& t : threads) {
